@@ -1,12 +1,22 @@
+import numpy as np
+
 import xobjects as xo
+import xpart as xp
 
-from ..particles import Particles
 
 class BeamInteraction:
+    def __init__(self, name=None, interaction_process=None,
+                 length=0, isthick=None):
 
-    def __init__(self, interaction_process):
+        self.name = name
         self.interaction_process = interaction_process
 
+        self.length = length
+
+        if isthick is None:
+            isthick = True if length > 0 else False
+        self.isthick = isthick
+
     def track(self, particles):
 
         assert isinstance(particles._buffer.context, xo.ContextCpu)
@@ -21,7 +31,7 @@ class BeamInteraction:
         if products is None or products['x'].size == 0:
             particles.reorganize()
         else:
-            new_particles = Particles(_context=particles._buffer.context,
+            new_particles = xp.Particles(_context=particles._buffer.context,
                     p0c = particles.p0c[0], # TODO: Should we check that 
                                             #       they are all the same?
                     s = products['s'],
@@ -33,6 +43,8 @@ class BeamInteraction:
                     delta = products['delta'],
                     mass_ratio = products['mass_ratio'],
                     charge_ratio = products['charge_ratio'],
+                    at_element = products['at_element'],
+                    at_turn = products['at_turn'],
                     parent_particle_id = products['parent_particle_id'])
 
             particles.add_particles(new_particles)
