@@ -101,20 +101,27 @@ class Elens(BeamElement):
                'inner_radius': xo.Float64,
                'outer_radius': xo.Float64,
                'elens_length': xo.Float64,
-               'voltage':      xo.Float64
+               'voltage':      xo.Float64,
+               'residual_kick_x': xo.Float64,
+               'residual_kick_y': xo.Float64
               }
 
-    def __init__(self,  inner_radius = None,
-                        outer_radius = None,
-                        current      = None,
-                        elens_length = None,
-                        voltage      = None, **kwargs):
+    def __init__(self,  inner_radius  = None,
+                        outer_radius  = None,
+                        current       = None,
+                        elens_length  = None,
+                        voltage       = None,
+                        residual_kick_x = 0,
+                        residual_kick_y = 0,
+                        **kwargs):
         super().__init__(**kwargs)
         self.inner_radius    = inner_radius
         self.outer_radius    = outer_radius
         self.current         = current
         self.elens_length    = elens_length
         self.voltage         = voltage
+        self.residual_kick_x   = residual_kick_x
+        self.residual_kick_y   = residual_kick_y
 
     def get_backtrack_element(self, _context=None, _buffer=None, _offset=None):
         return self.__class__(
@@ -166,6 +173,16 @@ class SRotation(BeamElement):
 SRotation.XoStruct.extra_sources = [
         _pkg_root.joinpath('beam_elements/elements_src/srotation.h')]
 
+
+def _update_bal_from_knl_ksl(knl, ksl, bal, context=None):
+    assert len(bal) == 2*len(knl) == 2*len(ksl)
+    idx = np.array([ii for ii in range(0, len(knl))])
+    inv_factorial = 1.0 / factorial(idx, exact=True)
+    if context is not None:
+        inv_factorial = context.nparray_to_context_array(inv_factorial)
+    bal[0::2] = knl * inv_factorial
+    bal[1::2] = ksl * inv_factorial
+
 class Multipole(BeamElement):
     '''Beam element modeling a thin magnetic multipole. Parameters:
 
@@ -226,10 +243,7 @@ class Multipole(BeamElement):
             order = n - 1
             bal = np.zeros(2 * order + 2)
 
-            idx = np.array([ii for ii in range(0, len(knl))])
-            inv_factorial = 1.0 / factorial(idx, exact=True)
-            bal[0::2] = knl * inv_factorial
-            bal[1::2] = ksl * inv_factorial
+            _update_bal_from_knl_ksl(knl, ksl, bal)
 
             kwargs["bal"] = bal
             kwargs["order"] = order
@@ -238,33 +252,53 @@ class Multipole(BeamElement):
             kwargs["bal"] = bal
             kwargs["order"] = (len(bal) - 2) // 2
 
-        ## TODO: Remove when xobjects is fixed
-        if 'bal' in kwargs.keys():
-            keep_bal = kwargs['bal']
-            kwargs['bal'] = len(keep_bal) # Initializes with zeros
-
         self.xoinitialize(**kwargs)
 
-        ## TODO: Remove when xobjects is fixed
-        if 'bal' in kwargs.keys():
-            if isinstance(keep_bal, np.ndarray):
-                keep_bal = self._buffer.context.nparray_to_context_array(
-                                                                    keep_bal)
-            self.bal[:]= keep_bal # Sets the values
-
     @property
     def knl(self):
         bal_length = len(self.bal)
         idxes = np.array([ii for ii in range(0, bal_length, 2)])
-        return [self.bal[idx] * factorial(idx // 2, exact=True) for idx in idxes]
+        _bal = self._buffer.context.nparray_from_context_array(self.bal)
+        _knl = self._buffer.context.nparray_to_context_array(np.array(
+            [_bal[idx] * factorial(idx // 2, exact=True) for idx in idxes]))
+        return self._buffer.context.linked_array_type.from_array(
+                                        _knl,
+                                        mode='setitem_from_container',
+                                        container=self,
+                                        container_setitem_name='_knl_setitem')
+
+    @knl.setter
+    def knl(self, value):
+        self.knl[:] = value
+
+    def _knl_setitem(self, indx, val):
+        _knl = self.knl.copy()
+        _knl[indx] = val
+        _update_bal_from_knl_ksl(_knl, self.ksl, self.bal,
+                                 context=self._buffer.context)
 
     @property
     def ksl(self):
         bal_length = len(self.bal)
         idxes = np.array([ii for ii in range(0, bal_length, 2)])
-        return [self.bal[idx + 1] * factorial(idx // 2, exact=True) for idx in idxes]
-        #idx = np.array([ii for ii in range(0, len(self.bal), 2)])
-        #return self.bal[idx + 1] * factorial(idx // 2, exact=True)
+        _bal = self._buffer.context.nparray_from_context_array(self.bal)
+        _ksl = self._buffer.context.nparray_to_context_array(np.array(
+            [_bal[idx + 1] * factorial(idx // 2, exact=True) for idx in idxes]))
+        return self._buffer.context.linked_array_type.from_array(
+                                        _ksl,
+                                        mode='setitem_from_container',
+                                        container=self,
+                                        container_setitem_name='_ksl_setitem')
+
+    @ksl.setter
+    def ksl(self, value):
+        self.ksl[:] = value
+
+    def _ksl_setitem(self, indx, val):
+        _ksl = self.ksl.copy()
+        _ksl[indx] = val
+        _update_bal_from_knl_ksl(self.knl, _ksl, self.bal,
+                                 context=self._buffer.context)
 
     def get_backtrack_element(self, _context=None, _buffer=None, _offset=None):
         return self.__class__(
@@ -277,10 +311,10 @@ class Multipole(BeamElement):
                               _context=_context, _buffer=_buffer, _offset=_offset)
 
 Multipole.XoStruct.extra_sources = [
-    xp.general._pkg_root.joinpath(
-        'random_number_generator/rng_src/base_rng.h'),
-    xp.general._pkg_root.joinpath(
-        'random_number_generator/rng_src/local_particle_rng.h'),
+    xp.general._pkg_root.joinpath('random_number_generator/rng_src/base_rng.h'),
+    xp.general._pkg_root.joinpath('random_number_generator/rng_src/local_particle_rng.h'),
+    _pkg_root.joinpath('headers/constants.h'),
+    _pkg_root.joinpath('headers/synrad_spectrum.h'),
     _pkg_root.joinpath('beam_elements/elements_src/multipole.h')]
 
 
@@ -402,11 +436,11 @@ class RFMultipole(BeamElement):
 
         elif (
             bal is not None
-            and bal
+            #and bal
             and len(bal) >= 2
             and ((len(bal) % 2) == 0)
             and phase is not None
-            and phase
+            #and phase
             and len(phase) >= 2
             and ((len(phase) % 2) == 0)
         ):
@@ -434,41 +468,19 @@ class RFMultipole(BeamElement):
             self.bal[:] = ctx.nparray_to_context_array(temp_bal)
             self.phase[:] = ctx.nparray_to_context_array(temp_phase)
 
-    @property
-    def knl(self):
-        idx = np.array([ii for ii in range(0, len(self.bal), 2)])
-        return self.bal[idx] * factorial(idx // 2, exact=True)
-
-    @property
-    def ksl(self):
-        idx = np.array([ii for ii in range(0, len(self.bal), 2)])
-        return self.bal[idx + 1] * factorial(idx // 2, exact=True)
-
-    def set_knl(self, value, order):
-        assert order <= self.order
-        self.bal[order * 2] = value / factorial(order, exact=True)
-
-    def set_ksl(self, value, order):
-        assert order <= self.order
-        self.bal[order * 2 + 1] = value / factorial(order, exact=True)
-
     @property
     def pn(self):
-        idx = np.array([ii for ii in range(0, len(self.phase), 2)])
-        return self.phase[idx]
+        _pn = self._buffer.context.nparray_to_context_array(np.array(
+            [self._xobject.phase[ii] for ii in range(0, len(self.phase), 2)]))
+        return self._buffer.context.linked_array_type.from_array(
+                                        _pn, mode='readonly')
 
     @property
     def ps(self):
-        idx = np.array([ii for ii in range(0, len(self.phase), 2)])
-        return self.phase[idx + 1]
-
-    def set_pn(self, value, order):
-        assert order <= self.order
-        self.phase[order * 2] = value
-
-    def set_ps(self, value, order):
-        assert order <= self.order
-        self.phase[order * 2 + 1] = value
+        _ps = self._buffer.context.nparray_to_context_array(np.array(
+            [self._xobject.phase[ii+1] for ii in range(0, len(self.phase), 2)]))
+        return self._buffer.context.linked_array_type.from_array(
+                                        _ps, mode='readonly')
 
     def get_backtrack_element(self, _context=None, _buffer=None, _offset=None):
         return self.__class__(
@@ -480,6 +492,11 @@ class RFMultipole(BeamElement):
                               p = [pp for pp in self.phase],
                               _context=_context, _buffer=_buffer, _offset=_offset)
 
+RFMultipole.knl = Multipole.knl
+RFMultipole.ksl = Multipole.ksl
+RFMultipole._knl_setitem = Multipole._knl_setitem
+RFMultipole._ksl_setitem = Multipole._ksl_setitem
+
 RFMultipole.XoStruct.extra_sources = [
         _pkg_root.joinpath('headers/constants.h'),
         _pkg_root.joinpath('beam_elements/elements_src/rfmultipole.h')]
