@@ -1,10 +1,14 @@
 from pathlib import Path
 import numpy as np
-from scipy.optimize import fsolve
+import logging
 
 from .general import _pkg_root
 from .line_frozen import LineFrozen
 from .base_element import _handle_per_particle_blocks
+from .twiss_from_tracker import (twiss_from_tracker,
+                                 compute_one_turn_matrix_finite_differences,
+                                 find_closed_orbit,
+                                )
 
 import xobjects as xo
 import xpart as xp
@@ -12,6 +16,8 @@ import xpart as xp
 from .beam_elements import Drift
 from .line import Line
 
+logger = logging.getLogger(__name__)
+
 def _check_is_collective(ele):
     iscoll = not hasattr(ele, 'iscollective') or ele.iscollective
     return iscoll
@@ -92,8 +98,6 @@ class Tracker:
     ):
 
         assert _offset is None
-        assert track_kernel is None
-        assert element_classes is None
 
         self.skip_end_turn_actions = skip_end_turn_actions
         self.particles_class = particles_class
@@ -123,7 +127,7 @@ class Tracker:
             this_part.iscollective=False
             parts.append(this_part)
 
-        # Transform non collective elements into xtrack elements 
+        # Transform non collective elements into xtrack elements
         noncollective_xelements = []
         for ii, pp in enumerate(parts):
             if not _check_is_collective(pp):
@@ -142,26 +146,29 @@ class Tracker:
 
         # Build tracker for all non collective elements
         supertracker = Tracker(_buffer=_buffer,
-                line=Line(elements=noncollective_xelements),
-                    particles_class=particles_class,
-                    particles_monitor_class=particles_monitor_class,
-                    global_xy_limit=global_xy_limit,
-                    local_particle_src=local_particle_src,
-                    save_source_as=save_source_as
-                    )
+                line=Line(elements=noncollective_xelements,
+                          element_names=line.element_names),
+                track_kernel=track_kernel,
+                element_classes=element_classes,
+                particles_class=particles_class,
+                particles_monitor_class=particles_monitor_class,
+                global_xy_limit=global_xy_limit,
+                local_particle_src=local_particle_src,
+                save_source_as=save_source_as
+                )
 
         # Build trackers for non collective parts
         for ii, pp in enumerate(parts):
             if not _check_is_collective(pp):
                 parts[ii] = Tracker(_buffer=_buffer,
-                                    line=pp,
-                                    element_classes=supertracker.element_classes,
-                                    track_kernel=supertracker.track_kernel,
-                                    particles_class=particles_class,
-                                    particles_monitor_class=particles_monitor_class,
-                                    global_xy_limit=global_xy_limit,
-                                    local_particle_src=local_particle_src,
-                                    skip_end_turn_actions=True)
+                                line=pp,
+                                element_classes=supertracker.element_classes,
+                                track_kernel=supertracker.track_kernel,
+                                particles_class=particles_class,
+                                particles_monitor_class=particles_monitor_class,
+                                global_xy_limit=global_xy_limit,
+                                local_particle_src=local_particle_src,
+                                skip_end_turn_actions=True)
 
         # Make a "marker" element to increase at_element
         self._zerodrift = Drift(_context=_buffer.context, length=0)
@@ -202,7 +209,8 @@ class Tracker:
 
         self.global_xy_limit = global_xy_limit
 
-        frozenline = LineFrozen(_context=_context, _buffer=_buffer, _offset=_offset,
+        frozenline = LineFrozen(
+                    _context=_context, _buffer=_buffer, _offset=_offset,
                     line=line)
 
         context = frozenline._buffer.context
@@ -221,11 +229,11 @@ class Tracker:
         line._freeze()
         self.line = line
         self._line_frozen = frozenline
-        ele_offsets = np.array([ee._offset for ee in frozenline.elements], dtype=np.int64)
+        ele_offsets = np.array(
+            [ee._offset for ee in frozenline.elements], dtype=np.int64)
         ele_typeids = np.array(
-            [element_classes.index(ee._xobject.__class__) for ee in frozenline.elements],
-            dtype=np.int64,
-        )
+            [element_classes.index(ee._xobject.__class__)
+                for ee in frozenline.elements], dtype=np.int64)
         ele_offsets_dev = context.nparray_to_context_array(ele_offsets)
         ele_typeids_dev = context.nparray_to_context_array(ele_typeids)
 
@@ -249,60 +257,87 @@ class Tracker:
 
         self.track=self._track_no_collective
 
-    def find_closed_orbit(self, particle_co_guess):
-
-        particle_co_guess = particle_co_guess.copy(
-                            _context=self._buffer.context)
-
-        res = fsolve(lambda p: p - _one_turn_map(p, particle_co_guess, self),
-              x0=np.array([particle_co_guess._xobject.x[0],
-                           particle_co_guess._xobject.px[0],
-                           particle_co_guess._xobject.y[0],
-                           particle_co_guess._xobject.py[0],
-                           particle_co_guess._xobject.zeta[0],
-                           particle_co_guess._xobject.delta[0]]))
-
-        particle_on_co = particle_co_guess.copy()
-        particle_on_co.x = res[0]
-        particle_on_co.px = res[1]
-        particle_on_co.y = res[2]
-        particle_on_co.py = res[3]
-        particle_on_co.zeta = res[4]
-        particle_on_co.update_delta(res[5])
-
-        return particle_on_co
+    def find_closed_orbit(self, particle_co_guess=None, particle_ref=None,
+                          co_search_settings={}):
+        return find_closed_orbit(self, particle_co_guess=particle_co_guess,
+                                 particle_ref=particle_ref,
+                                 co_search_settings=co_search_settings)
 
     def compute_one_turn_matrix_finite_differences(
             self, particle_on_co,
-            dx=1e-7, dpx=1e-10, dy=1e-7, dpy=1e-10,
-            dzeta=1e-6, ddelta=1e-7):
+            steps_r_matrix=None):
+        return compute_one_turn_matrix_finite_differences(self, particle_on_co,
+                                                   steps_r_matrix)
 
-        particle_on_co = particle_on_co.copy(
-                            _context=self._buffer.context)
-        # Find R matrix
-        p0 = np.array([
-               particle_on_co._xobject.x[0],
-               particle_on_co._xobject.px[0],
-               particle_on_co._xobject.y[0],
-               particle_on_co._xobject.py[0],
-               particle_on_co._xobject.zeta[0],
-               particle_on_co._xobject.delta[0]])
-        II = np.eye(6)
-        RR = np.zeros((6, 6), dtype=np.float64)
-        for jj, dd in enumerate([dx, dpx, dy, dpy, dzeta, ddelta]):
-            RR[:,jj]=(_one_turn_map(p0+II[jj]*dd, particle_on_co, self)-
-                      _one_turn_map(p0-II[jj]*dd, particle_on_co, self))/(2*dd)
+    def twiss(self, particle_ref, r_sigma=0.01,
+        nemitt_x=1e-6, nemitt_y=1e-6,
+        n_theta=1000, delta_disp=1e-5, delta_chrom=1e-4,
+        particle_co_guess=None, steps_r_matrix=None,
+        co_search_settings=None, at_elements=None,
+        ):
 
-        return RR
+        if self.iscollective:
+            logger.warning(
+                'The tracker has collective elements.\n'
+                'In the twiss computation collective elements are'
+                ' replaced by drifts')
+            tracker = self._supertracker
+        else:
+            tracker = self
 
 
+        return twiss_from_tracker(tracker, particle_ref, r_sigma=r_sigma,
+            nemitt_x=nemitt_x, nemitt_y=nemitt_y,
+            n_theta=n_theta, delta_disp=delta_disp, delta_chrom=delta_chrom,
+            particle_co_guess=particle_co_guess,
+            steps_r_matrix=steps_r_matrix,
+            co_search_settings=co_search_settings,
+            at_elements=at_elements)
+
+
+    def filter_elements(self, mask=None, exclude_types_starting_with=None):
+        return self.__class__(
+                 _buffer=self._buffer,
+                 line=self.line.filter_elements(mask=mask,
+                     exclude_types_starting_with=exclude_types_starting_with),
+                 track_kernel=(self.track_kernel if not self.iscollective
+                                    else self._supertracker.track_kernel),
+                 element_classes=(self.element_classes if not self.iscollective
+                                    else self._supertracker.element_classes))
+
+    def cycle(self, index_first_element=None, name_first_element=None,
+              _buffer=None, _context=None):
+
+        cline = self.line.cycle(index_first_element=index_first_element,
+                                name_first_element=name_first_element)
+
+        if _buffer is None:
+            if _context is None:
+                _buffer = self._buffer
+            else:
+                _buffer = _context.new_buffer()
+
+        return self.__class__(
+                _buffer=_buffer,
+                line=cline,
+                track_kernel=(self.track_kernel if not self.iscollective
+                                    else self._supertracker.track_kernel),
+                element_classes=(self.element_classes if not self.iscollective
+                                    else self._supertracker.element_classes),
+                particles_class=self.particles_class,
+                skip_end_turn_actions=self.skip_end_turn_actions,
+                particles_monitor_class=self.particles_monitor_class,
+                global_xy_limit=self.global_xy_limit,
+                local_particle_src=self.local_particle_src,
+            )
+
     def get_backtracker(self, _context=None, _buffer=None):
 
         assert not self.iscollective
 
         if _buffer is None:
             if _context is None:
-                _context = xo.context_default
+                _context = self._buffer.context
             _buffer = _context.new_buffer()
 
         line = Line(elements=[], element_names=[])
@@ -588,28 +623,9 @@ class Tracker:
 
     def _slow_track_ebe(self,part):
         out=[]
-        for ii in range(len(line.elements)):
+        for ii in range(len(self.line.elements)):
             out.append(part.copy())
             self.track(part,ele_start=ii,num_elements=1)
         return out
 
-def _one_turn_map(p, particle_ref, tracker):
-    part = particle_ref.copy()
-    part.x = p[0]
-    part.px = p[1]
-    part.y = p[2]
-    part.py = p[3]
-    part.zeta = p[4]
-    part.update_delta(p[5])
-
-    tracker.track(part)
-    p_res = np.array([
-           part._xobject.x[0],
-           part._xobject.px[0],
-           part._xobject.y[0],
-           part._xobject.py[0],
-           part._xobject.zeta[0],
-           part._xobject.delta[0]])
-    return p_res
-
 
